@@ -151,7 +151,7 @@ def pairwise(data, varnames, vars, cause_list, cache_data, cache_weight, method)
                     cache_data[W] = data[~np.isnan(data[:, W_ids]).any(axis=1)]
             else:
                 cache_data[W] = data
-            cache_weight[W] = np.ones(len(cache_data[W]))
+            cache_weight[W] = np.ones(len(cache_data[W])) * data.shape[0] / len(cache_data[W])
     elif method == 'ipw':
         W = list(vars)
         while True:
@@ -169,7 +169,8 @@ def pairwise(data, varnames, vars, cause_list, cache_data, cache_weight, method)
                     cache_data[W] = data[data[:, W_ids].min(axis=1) >= 0]
                 else:
                     cache_data[W] = data[~np.isnan(data[:, W_ids]).any(axis=1)]
-                cache_weight[W] = compute_weights(data, varnames, W_ids, cause_list)
+                cache_weight[W] = compute_weights(data, varnames, W_ids, cause_list) / cache_data[W].shape[0] * \
+                                  data.shape[0]
             else:
                 cache_data[W] = data
                 cache_weight[W] = np.ones(len(data))
@@ -188,9 +189,10 @@ def pairwise(data, varnames, vars, cause_list, cache_data, cache_weight, method)
                     cache_data[W] = data[~np.isnan(data[:, W_ids]).any(axis=1)]
 
                 if pa_d:
-                    cache_weight[W] = np.ones(cache_data[W].shape[0])
+                    cache_weight[W] = np.ones(cache_data[W].shape[0]) / cache_data[W].shape[0] * data.shape[0]
                 else:
-                    cache_weight[W] = compute_weights(data, varnames, W_ids, cause_list)
+                    cache_weight[W] = compute_weights(data, varnames, W_ids, cause_list) / cache_data[W].shape[0] * \
+                                      data.shape[0]
             else:
                 cache_data[W] = data
                 cache_weight[W] = np.ones(data.shape[0])
@@ -202,6 +204,8 @@ def pairwise(data, varnames, vars, cause_list, cache_data, cache_weight, method)
 # find the missing mechanism of dataset with missing values
 def find_causes(data, test_function='default', alpha=0.01):
     var_miss = data.columns[data.isnull().any()]
+    if len(var_miss) == 0:
+        return {}
     if all(data[var].dtype.name == 'category' for var in data):
         factor = True
         data = data.apply(lambda x: x.cat.codes)
